@@ -64,7 +64,7 @@ get_title_thingies(GntBox *box, char *title, int *p, int *r)
 	GntWidget *widget = GNT_WIDGET(box);
 	int len;
 	char *end = (char*)gnt_util_onscreen_width_to_pointer(title, widget->priv.width - 4, &len);
-	
+
 	if (p)
 		*p = (widget->priv.width - len) / 2;
 	if (r)
@@ -232,8 +232,8 @@ gnt_box_size_request(GntWidget *widget)
 				h = maxh;
 		}
 
-		gnt_widget_confirm_size(wid, w, h);
-		gnt_widget_set_size(wid, w, h);
+		if (gnt_widget_confirm_size(wid, w, h))
+			gnt_widget_set_size(wid, w, h);
 	}
 
 	reposition_children(widget);
@@ -422,7 +422,9 @@ gnt_box_confirm_size(GntWidget *widget, int width, int height)
 
 		gnt_widget_get_size(wid, &w, &h);
 
-		if (wid != last && !child && w > 0 && h > 0 && gnt_widget_confirm_size(wid, w - wchange, h - hchange)) {
+		if (wid != last && !child && w > 0 && h > 0 &&
+				!GNT_WIDGET_IS_FLAG_SET(wid, GNT_WIDGET_INVISIBLE) &&
+				gnt_widget_confirm_size(wid, w - wchange, h - hchange)) {
 			child = wid;
 			break;
 		}
