@@ -28,6 +28,7 @@
 #include "dbus-maybe.h"
 #include "debug.h"
 #include "notify.h"
+#include "pounce.h"
 #include "prefs.h"
 #include "privacy.h"
 #include "prpl.h"
@@ -278,7 +279,8 @@ group_to_xmlnode(PurpleBlistNode *gnode)
 	group = (PurpleGroup *)gnode;
 
 	node = xmlnode_new("group");
-	xmlnode_set_attrib(node, "name", group->name);
+	if (g_strcmp0(group->name, _("Buddies")) != 0)
+		xmlnode_set_attrib(node, "name", group->name);
 
 	/* Write settings */
 	g_hash_table_foreach(group->node.settings, value_to_xmlnode, node);
@@ -2184,6 +2186,9 @@ void purple_blist_remove_buddy(PurpleBuddy *buddy)
 	if (ops && ops->remove_node)
 		ops->remove_node(node);
 
+	/* Remove this buddy's pounces */
+	purple_pounce_destroy_all_by_buddy(buddy);
+
 	/* Signal that the buddy has been removed before freeing the memory for it */
 	purple_signal_emit(purple_blist_get_handle(), "buddy-removed", buddy);
 
